@@ -1,7 +1,7 @@
 ﻿/*
  * ct_main_win.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -52,8 +52,8 @@ CtMainWin::CtMainWin(bool                            no_gui,
     set_icon(_pGtkIconTheme->load_icon(CtConst::APP_NAME, 48));
 
     _uCtActions.reset(new CtActions{this});
-    _uCtMenu.reset(new CtMenu{pCtConfig, _uCtActions.get()});
-    _uCtPrint.reset(new CtPrint{});
+    _uCtMenu.reset(new CtMenu{this});
+    _uCtPrint.reset(new CtPrint{this});
     _uCtStorage.reset(CtStorageControl::create_dummy_storage(this));
 
     _scrolledwindowTree.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
@@ -63,16 +63,21 @@ CtMainWin::CtMainWin(bool                            no_gui,
     _vboxText.pack_start(_init_window_header(), false, false);
     _vboxText.pack_start(_scrolledwindowText);
     if (_pCtConfig->treeRightSide) {
-        _hPaned.add1(_vboxText);
-        _hPaned.add2(_scrolledwindowTree);
+        _hPaned.pack1(_vboxText, Gtk::EXPAND);
+        _hPaned.pack2(_scrolledwindowTree, Gtk::FILL);
     }
     else {
-        _hPaned.add1(_scrolledwindowTree);
-        _hPaned.add2(_vboxText);
+        _hPaned.pack1(_scrolledwindowTree, Gtk::FILL);
+        _hPaned.pack2(_vboxText, Gtk::EXPAND);
     }
     _hPaned.property_wide_handle() = true;
+    _vPaned.pack1(_hPaned, Gtk::EXPAND);
+    _vPaned.pack2(_hBoxVte, Gtk::FILL);
+    _vPaned.property_wide_handle() = true;
 
     _pMenuBar = _uCtMenu->build_menubar();
+    _pScrolledWindowMenuBar = Gtk::manage(new Gtk::ScrolledWindow{});
+    _pScrolledWindowMenuBar->add(*_pMenuBar);
     _pMenuBar->set_name("MenuBar");
     _pBookmarksSubmenus[0] = CtMenu::find_menu_item(_pMenuBar, "BookmarksSubMenu");
     auto pPopumMenuTree = _uCtMenu->get_popup_menu(CtMenu::POPUP_MENU_TYPE::Node);
@@ -90,11 +95,23 @@ CtMainWin::CtMainWin(bool                            no_gui,
     add_accel_group(_uCtMenu->get_accel_group());
     _pToolbars = _uCtMenu->build_toolbars(_pRecentDocsMenuToolButton);
 
-    _vboxMain.pack_start(*_pMenuBar, false, false);
+    if (_pCtConfig->menubarInTitlebar) {
+        _pHeaderBar = Gtk::manage(new Gtk::HeaderBar{});
+        _pHeaderBar->set_has_subtitle(false);
+        _pHeaderBar->set_show_close_button(true);
+        _pHeaderBar->pack_start(*Gtk::manage(new Gtk::Label{" "}));
+        _pHeaderBar->pack_start(*_pScrolledWindowMenuBar);
+        _pHeaderBar->pack_start(*Gtk::manage(new Gtk::Label{" "}));
+        _pHeaderBar->show_all();
+        set_titlebar(*_pHeaderBar);
+    }
+    else {
+        _vboxMain.pack_start(*_pScrolledWindowMenuBar, false, false);
+    }
     for (auto pToolbar : _pToolbars) {
         _vboxMain.pack_start(*pToolbar, false, false);
     }
-    _vboxMain.pack_start(_hPaned);
+    _vboxMain.pack_start(_vPaned);
     _vboxMain.pack_start(_init_status_bar(), false, false);
     _vboxMain.show_all();
     add(_vboxMain);
@@ -143,7 +160,7 @@ CtMainWin::CtMainWin(bool                            no_gui,
         set_visible(false);
     }
     else {
-        if (_pCtConfig->systrayOn and _pCtConfig->startOnSystray) {
+        if (start_on_systray_is_active()) {
             /* Calling the 'present()' function apparently sets up selected
                node visibility within the TreeView panel, whereas this
                node setup is skipped when only calling 'set_visible(false)'.
@@ -160,15 +177,20 @@ CtMainWin::CtMainWin(bool                            no_gui,
         else {
             present();
         }
-        _hPaned.property_position() = _pCtConfig->hpanedPos; // must be after present() (#1534)
-        _ctTextview.signal_size_allocate().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_size_allocate));
-        signal_configure_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_window_configure_event), false);
+        Glib::signal_idle().connect_once([this](){
+            _hPaned.property_position() = _pCtConfig->hpanedPos; // must be after present() + process events pending (#1534, #1918, #2126)
+            _vPaned.property_position() = _pCtConfig->vpanedPos;
+            _ctTextview.signal_size_allocate().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_size_allocate));
+            signal_configure_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_window_configure_event), false);
 
-        // show status icon if needed
-        if (_pCtConfig->systrayOn) {
-            get_status_icon()->set_visible(true);
-        }
+            // show status icon if needed
+            if (_pCtConfig->systrayOn) {
+                get_status_icon()->set_visible(true);
+            }
+        });
     }
+
+    dispatcherErrorMsg.connect(sigc::mem_fun(*this, &CtMainWin::_on_dispatcher_error_msg));
 }
 
 CtMainWin::~CtMainWin()
@@ -178,6 +200,41 @@ CtMainWin::~CtMainWin()
     //std::cout << "~CtMainWin" << std::endl;
 }
 
+void CtMainWin::_on_dispatcher_error_msg()
+{
+    const std::string eroor_msg = errorsDEQueue.pop_front();
+    CtDialogs::error_dialog(eroor_msg, *this);
+}
+
+bool CtMainWin::start_on_systray_is_active() const
+{
+    return _pCtConfig->systrayOn and _pCtConfig->startOnSystray;
+}
+
+void CtMainWin::start_on_systray_delayed_file_open_set(const std::string& filepath, const std::string& nodename, const std::string& anchorname)
+{
+    _startOnSystray_delayedFilepath = filepath;
+    _startOnSystray_delayedNodeName = nodename;
+    _startOnSystray_delayedAnchorName = anchorname;
+}
+
+bool CtMainWin::start_on_systray_delayed_file_open_kick()
+{
+    if (not _startOnSystray_delayedFilepath.empty()) {
+        const std::string startOnSystray_delayedFilepath = _startOnSystray_delayedFilepath;
+        const std::string startOnSystray_delayedNodeName = _startOnSystray_delayedNodeName;
+        const std::string startOnSystray_delayedAnchorName = _startOnSystray_delayedAnchorName;
+        _startOnSystray_delayedFilepath.clear();
+        _startOnSystray_delayedNodeName.clear();
+        _startOnSystray_delayedAnchorName.clear();
+        if (file_open(startOnSystray_delayedFilepath, startOnSystray_delayedNodeName, startOnSystray_delayedAnchorName)) {
+            return true;
+        }
+        spdlog::warn("%s Couldn't open file: %s", __FUNCTION__, _startOnSystray_delayedFilepath);
+    }
+    return false;
+}
+
 std::string CtMainWin::get_code_icon_name(std::string code_type)
 {
     for (const auto& iconPair : CtConst::NODE_CODE_ICONS) {
@@ -185,14 +242,14 @@ std::string CtMainWin::get_code_icon_name(std::string code_type)
             return iconPair.second;
         }
     }
-    return CtConst::NODE_CUSTOM_ICONS.at(CtConst::NODE_ICON_CODE_ID);
+    return CtStockIcon::at(CtConst::NODE_ICON_CODE_ID);
 }
 
-Gtk::Image* CtMainWin::new_image_from_stock(const std::string& stockImage, Gtk::BuiltinIconSize size)
+Gtk::Image* CtMainWin::new_managed_image_from_stock(const std::string& stockImage, Gtk::BuiltinIconSize size)
 {
-    Gtk::Image* image = Gtk::manage(new Gtk::Image{});
-    image->set_from_icon_name(stockImage, size);
-    return image;
+    auto pImage = Gtk::manage(new Gtk::Image{});
+    pImage->set_from_icon_name(stockImage, size);
+    return pImage;
 }
 
 void CtMainWin::_reset_CtTreestore_CtTreeview()
@@ -210,6 +267,7 @@ void CtMainWin::_reset_CtTreestore_CtTreeview()
     _uCtTreestore->tree_view_connect(_uCtTreeview.get());
     _uCtTreeview->set_tree_node_name_wrap_width(_pCtConfig->cherryWrapEnabled, _pCtConfig->cherryWrapWidth);
     _uCtTreeview->get_column(CtTreeView::AUX_ICON_COL_NUM)->set_visible(!_pCtConfig->auxIconHide);
+    show_hide_tree_lines(_pCtConfig->treeLinesVisible);
 
     _tree_just_auto_expanded = false;
     _uCtTreeview->signal_cursor_changed().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_cursor_changed));
@@ -244,11 +302,15 @@ void CtMainWin::config_apply()
     }
 
     show_hide_tree_view(_pCtConfig->treeVisible);
+    (void)show_hide_vte_cmd_passed_as_first_in_session(_pCtConfig->vteVisible, nullptr/*first_cmd_passed*/);
+    show_hide_menubar(_pCtConfig->menubarVisible);
     show_hide_win_header(_pCtConfig->showNodeNameHeader);
     _ctWinHeader.lockIcon.hide();
     _ctWinHeader.bookmarkIcon.hide();
+    _ctWinHeader.ghostIcon.hide();
 
     menu_rebuild_toolbars(false);
+    show_hide_statusbar(_pCtConfig->statusbarVisible);
 
     _ctStatusBar.progressBar.hide();
     _ctStatusBar.stopButton.hide();
@@ -264,6 +326,9 @@ void CtMainWin::config_apply()
     _ctTextview.set_pixels_inside_wrap(_pCtConfig->spaceAroundLines, _pCtConfig->relativeWrappedSpace);
     _ctTextview.set_wrap_mode(_pCtConfig->lineWrapping ? Gtk::WrapMode::WRAP_WORD_CHAR : Gtk::WrapMode::WRAP_NONE);
 
+    if (2 != _pCtConfig->cursorBlink) {
+        Gtk::Settings::get_default()->property_gtk_cursor_blink() = _pCtConfig->cursorBlink;
+    }
     update_theme();
 }
 
@@ -301,12 +366,13 @@ void CtMainWin::update_theme()
         css_str += ".ct-view-panel { padding-bottom: 400px } ";
     }
     css_str += ".ct-codebox.ct-view-rich-text" + rtFont;
-    css_str += ".ct-codebox.ct-view-plain-text" + codeFont;
+    css_str += ".ct-codebox.ct-view-plain-text" + plFont;
     css_str += ".ct-codebox.ct-view-code" + codeFont;
     css_str += ".ct-tree-panel" + treeFont;
+    css_str += ".ct-table-light" + rtFont;
     css_str += " ";
     css_str += ".ct-tree-panel { color: " + _pCtConfig->ttDefFg + "; background-color: " + _pCtConfig->ttDefBg + "; } ";
-    css_str += ".ct-tree-panel:selected { color: " + _pCtConfig->ttDefBg + "; background: #5294e2; } ";
+    css_str += ".ct-tree-panel:selected { color: " + _pCtConfig->ttSelFg + "; background: " + _pCtConfig->ttSelBg + "; } ";
     css_str += ".ct-tree-scroll-panel { background-color: " + _pCtConfig->ttDefBg + "; } ";
     css_str += ".ct-header-panel { background-color: " + _pCtConfig->ttDefBg + "; } ";
     css_str += ".ct-header-panel button { margin: 2px; padding: 0 4px 0 4px; } ";
@@ -315,6 +381,7 @@ void CtMainWin::update_theme()
     css_str += ".ct-table grid { background: #cccccc; border-style:solid; border-width: 1px; border-color: gray; } ";
     css_str += "toolbar { padding: 2px 2px 2px 2px; } ";
     css_str += "toolbar button { padding: 0px; } ";
+    css_str += "textview border { background-color: transparent; } "; // Loss of transparency with PNGs (#1402, #2132)
     //printf("css_str_len=%zu\n", css_str.size());
 
     if (_css_provider_theme) {
@@ -361,10 +428,13 @@ Gtk::EventBox& CtMainWin::_init_window_header()
     _ctWinHeader.lockIcon.hide();
     _ctWinHeader.bookmarkIcon.set_from_icon_name("ct_pin", Gtk::ICON_SIZE_MENU);
     _ctWinHeader.bookmarkIcon.hide();
+    _ctWinHeader.ghostIcon.set_from_icon_name("ct_ghost", Gtk::ICON_SIZE_MENU);
+    _ctWinHeader.ghostIcon.hide();
     _ctWinHeader.headerBox.pack_start(_ctWinHeader.buttonBox, false, false);
     _ctWinHeader.headerBox.pack_start(_ctWinHeader.nameLabel, true, true);
     _ctWinHeader.headerBox.pack_start(_ctWinHeader.lockIcon, false, false);
     _ctWinHeader.headerBox.pack_start(_ctWinHeader.bookmarkIcon, false, false);
+    _ctWinHeader.headerBox.pack_start(_ctWinHeader.ghostIcon, false, false);
     _ctWinHeader.eventBox.add(_ctWinHeader.headerBox);
     _ctWinHeader.eventBox.get_style_context()->add_class("ct-header-panel");
     return _ctWinHeader.eventBox;
@@ -380,7 +450,7 @@ void CtMainWin::window_header_update()
     std::string foreground = curr_tree_iter().get_node_foreground();
     foreground = foreground.empty() ? _pCtConfig->ttDefFg : foreground;
     _ctWinHeader.nameLabel.set_markup(
-        "<b><span foreground=\"" + foreground + "\" size=\"large\">" + str::xml_escape(name) + "</span></b>");
+        "<span foreground=\"" + foreground + "\" font_desc=\"" + _pCtConfig->treeFont + "\" font_weight=\"bold\">" + str::xml_escape(name) + "</span>");
 
     // update last visited buttons
     if (_pCtConfig->nodesOnNodeNameHeader == 0) {
@@ -413,7 +483,7 @@ void CtMainWin::window_header_update()
         for (auto iter = nodes.rbegin(); iter != nodes.rend(); ++iter) {
             if (*iter == curr_node) continue;
             if (CtTreeIter node = get_tree_store().get_node_from_node_id(*iter)) {
-                Glib::ustring name = "<small>" + str::xml_escape(node.get_node_name()) + "</small>";
+                Glib::ustring name = "<span font_desc=\"" + _pCtConfig->treeFont + "\">" + str::xml_escape(node.get_node_name()) + "</span>";
                 Glib::ustring tooltip = CtMiscUtil::get_node_hierarchical_name(node, "/", false);
                 if (auto button = dynamic_cast<Gtk::Button*>(buttons[button_idx])) {
                     if (auto label = dynamic_cast<Gtk::Label*>(button->get_child())) {
@@ -436,12 +506,17 @@ void CtMainWin::window_header_update()
     }
 }
 
-void CtMainWin::window_header_update_lock_icon(bool show)
+void CtMainWin::window_header_update_lock_icon(const bool show)
 {
     _ctWinHeader.lockIcon.set_visible(show);
 }
 
-void CtMainWin::window_header_update_bookmark_icon(bool show)
+void CtMainWin::window_header_update_ghost_icon(const bool show)
+{
+    _ctWinHeader.ghostIcon.set_visible(show);
+}
+
+void CtMainWin::window_header_update_bookmark_icon(const bool show)
 {
     _ctWinHeader.bookmarkIcon.set_visible(show);
 }
@@ -482,15 +557,18 @@ void CtMainWin::menu_set_bookmark_menu_items()
 
 void CtMainWin::menu_set_items_recent_documents()
 {
+    if (not _pCtConfig->rememberRecentDocs) {
+        return;
+    }
     sigc::slot<void, const std::string&> recent_doc_open_action = [&](const std::string& filepath){
-        if (Glib::file_test(filepath, Glib::FILE_TEST_IS_REGULAR)) {
-            if (file_open(filepath, "")) {
+        if (Glib::file_test(filepath, Glib::FILE_TEST_EXISTS)) {
+            if (file_open(filepath, ""/*node*/, ""/*anchor*/)) {
                 _pCtConfig->recentDocsFilepaths.move_or_push_front(fs_canonicalize_filename(filepath));
                 menu_set_items_recent_documents();
             }
         }
         else {
-            g_autofree gchar* title = g_strdup_printf(_("The Document %s was Not Found"), filepath.c_str());
+            g_autofree gchar* title = g_strdup_printf(_("The Path %s does Not Exist"), str::xml_escape(filepath).c_str());
             CtDialogs::error_dialog(Glib::ustring{title}, *this);
             _pCtConfig->recentDocsFilepaths.move_or_push_back(fs_canonicalize_filename(filepath));
             menu_set_items_recent_documents();
@@ -520,7 +598,7 @@ void CtMainWin::menu_set_items_recent_documents()
 void CtMainWin::menu_set_visible_exit_app(bool visible)
 {
     if (auto quit_label = CtMenu::get_accel_label(CtMenu::find_menu_item(_pMenuBar, "quit_app"))) {
-        quit_label->set_label(visible ? _("Hide") : _("Quit"));
+        quit_label->set_label(visible ? _("_Hide") : _("_Quit"));
         quit_label->set_tooltip_markup(visible ?  _("Hide the Window") : _("Quit the Application"));
     }
     CtMenu::find_menu_item(_pMenuBar, "exit_app")->set_visible(visible);
@@ -535,7 +613,12 @@ void CtMainWin::menu_rebuild_toolbars(bool new_toolbar)
         _pToolbars = _uCtMenu->build_toolbars(_pRecentDocsMenuToolButton);
         for (auto toolbar = _pToolbars.rbegin(); toolbar != _pToolbars.rend(); ++toolbar) {
             _vboxMain.pack_start(*(*toolbar), false, false);
-            _vboxMain.reorder_child(*(*toolbar), 1);
+            if (not _pCtConfig->menubarInTitlebar) {
+                _vboxMain.reorder_child(*(*toolbar), 1);
+            }
+            else {
+                _vboxMain.reorder_child(*(*toolbar), 0);
+            }
         }
         menu_set_items_recent_documents();
         for (auto pToolbar: _pToolbars) {
@@ -558,26 +641,26 @@ void CtMainWin::config_switch_tree_side()
     _hPaned.remove(_scrolledwindowTree);
     _hPaned.remove(_vboxText);
     if (_pCtConfig->treeRightSide) {
-        _hPaned.add1(_vboxText);
-        _hPaned.add2(_scrolledwindowTree);
+        _hPaned.pack1(_vboxText, Gtk::EXPAND);
+        _hPaned.pack2(_scrolledwindowTree, Gtk::FILL);
         _hPaned.property_position() = text_width;
     }
     else {
-        _hPaned.add1(_scrolledwindowTree);
-        _hPaned.add2(_vboxText);
+        _hPaned.pack1(_scrolledwindowTree, Gtk::FILL);
+        _hPaned.pack2(_vboxText, Gtk::EXPAND);
         _hPaned.property_position() = tree_width;
     }
+    _pCtConfig->hpanedPos = _hPaned.property_position();
 }
 
 void CtMainWin::_zoom_tree(bool is_increase)
 {
     Glib::RefPtr<Gtk::StyleContext> context = _uCtTreeview->get_style_context();
-    Pango::FontDescription fontDesc = context->get_font(context->get_state());
+    const Pango::FontDescription fontDesc = context->get_font(context->get_state());
     int size = fontDesc.get_size() / Pango::SCALE + (is_increase ? 1 : -1);
     if (size < 6) size = 6;
-    fontDesc.set_size(size * Pango::SCALE);
-    _pCtConfig->treeFont = CtFontUtil::get_font_str(fontDesc);
-    signal_app_apply_for_each_window([](CtMainWin* win) { win->update_theme(); });
+    _pCtConfig->treeFont = CtFontUtil::get_font_str(CtFontUtil::get_font_family(_pCtConfig->treeFont), size);
+    signal_app_apply_for_each_window([](CtMainWin* win) { win->update_theme(); win->window_header_update(); });
 }
 
 void CtMainWin::reset()
@@ -597,6 +680,7 @@ void CtMainWin::reset()
     }
     _ctWinHeader.nameLabel.set_markup("");
     window_header_update_lock_icon(false);
+    window_header_update_ghost_icon(false);
     window_header_update_bookmark_icon(false);
     menu_set_bookmark_menu_items();
     _uCtMenu->find_action("ct_vacuum")->signal_set_visible.emit(false);
@@ -650,3 +734,18 @@ void CtMainWin::update_selected_node_statusbar_info()
     }
     _ctStatusBar.update_status(statusbar_text);
 }
+
+void CtMainWin::tree_node_paste_from_other_window(CtMainWin* pWinToCopyFrom, gint64 nodeIdToCopyFrom)
+{
+    if (not pWinToCopyFrom) {
+        CtDialogs::warning_dialog(_("No Previous Node Copy Was Performed During This Session or the Source Tree is No Longer Available."), *this);
+        return;
+    }
+    CtTreeStore& other_ct_tree_store = pWinToCopyFrom->get_tree_store();
+    CtTreeIter   other_ct_tree_iter = other_ct_tree_store.get_node_from_node_id(nodeIdToCopyFrom);
+    if (not other_ct_tree_iter) {
+        CtDialogs::warning_dialog(_("The Source Tree Node is No Longer Available."), *this);
+        return;
+    }
+    _uCtActions->node_subnodes_paste2(other_ct_tree_iter, pWinToCopyFrom);
+}
