@@ -1,7 +1,7 @@
 /*
  * ct_menu.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -34,7 +34,10 @@ static xmlpp::Attribute* get_attribute(xmlpp::Node* pNode, char const* name)
 static void on_menu_activate(void* /*pObject*/, CtMenuAction* pAction)
 {
     if (pAction) {
-        pAction->run_action();
+        // this allows the menu to close before the action is executed
+        Glib::signal_idle().connect_once([pAction](){
+            pAction->run_action();
+        });
     }
 }
 
@@ -44,12 +47,13 @@ const std::string& CtMenuAction::get_shortcut(CtConfig* pCtConfig) const
     return it != pCtConfig->customKbShortcuts.end() ? it->second : built_in_shortcut;
 }
 
-CtMenu::CtMenu(CtConfig* pCtConfig, CtActions* pActions)
- : _pCtConfig{pCtConfig}
+CtMenu::CtMenu(CtMainWin* pCtMainWin)
+ : _pCtMainWin{pCtMainWin}
+ , _pCtConfig{pCtMainWin->get_ct_config()}
 {
     _pAccelGroup = Gtk::AccelGroup::create();
     _rGtkBuilder = Gtk::Builder::create();
-    init_actions(pActions);
+    init_actions(pCtMainWin->get_ct_actions());
 }
 
 /*static*/ Gtk::MenuItem* CtMenu::create_menu_item(Gtk::Menu* pMenu, const char* name, const char* image, const char* desc)
@@ -85,23 +89,6 @@ CtMenu::CtMenu(CtConfig* pCtConfig, CtActions* pActions)
     return nullptr;
 }
 
-/*static*/ int CtMenu::calculate_image_shift(Gtk::MenuItem* menuItem)
-{
-    if (menuItem)
-        if (auto box = dynamic_cast<Gtk::Box *>(menuItem->get_child()))
-            if (auto image = dynamic_cast<Gtk::Image *>(box->get_children()[0])) {
-                auto allocation_menuitem = menuItem->get_allocation();
-                auto allocation_image = image->get_allocation();
-
-                int shift = -allocation_image.get_x();
-                if (menuItem->get_direction() == Gtk::TEXT_DIR_RTL) {
-                    shift += (allocation_menuitem.get_width() - allocation_image.get_width());
-                }
-                return shift;
-            }
-    return 0;
-}
-
 std::vector<Gtk::Toolbar*> CtMenu::build_toolbars(Gtk::MenuToolButton*& pRecentDocsMenuToolButton)
 {
     pRecentDocsMenuToolButton = nullptr;
@@ -176,6 +163,7 @@ Gtk::Menu* CtMenu::get_popup_menu(POPUP_MENU_TYPE popupMenuType)
     if (_popupMenus[popupMenuType] == nullptr) {
         Gtk::Menu* pMenu = Gtk::manage(new Gtk::Menu{});
         build_popup_menu(pMenu, popupMenuType);
+        pMenu->attach_to_widget(*_pCtMainWin);
         _popupMenus[popupMenuType] = pMenu;
     }
     return _popupMenus[popupMenuType];
@@ -188,8 +176,10 @@ void CtMenu::build_popup_menu(Gtk::Menu* pMenu, POPUP_MENU_TYPE popupMenuType)
         case CtMenu::POPUP_MENU_TYPE::Text: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_text(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Code: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_code(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Image: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_image(), nullptr); break;
+        case CtMenu::POPUP_MENU_TYPE::Latex: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_latex(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Anchor: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_anchor(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::EmbFile: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_embfile(), nullptr); break;
+        case CtMenu::POPUP_MENU_TYPE::Terminal: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_terminal(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Link: {
             _add_menu_separator(pMenu);
             _add_menu_item(pMenu, find_action("apply_tag_link"));
@@ -217,7 +207,11 @@ void CtMenu::build_popup_menu(Gtk::Menu* pMenu, POPUP_MENU_TYPE popupMenuType)
             _add_menu_item(pMenu, find_action("codebox_increase_height"));
             _add_menu_item(pMenu, find_action("codebox_decrease_height"));
             _add_menu_separator(pMenu);
-            _add_menu_item(pMenu, find_action("exec_code"));
+            _add_menu_item(pMenu, find_action("exec_code_all"));
+            _add_menu_item(pMenu, find_action("exec_code_los"));
+            _add_menu_item(pMenu, find_action("strip_trail_spaces"));
+            _add_menu_item(pMenu, find_action("repl_tabs_spaces"));
+            _add_menu_separator(pMenu);
             _add_menu_item(pMenu, find_action("codebox_change_properties"));
         } break;
         case CtMenu::POPUP_MENU_TYPE::PopupMenuNum: {
@@ -253,8 +247,8 @@ void CtMenu::build_popup_menu_table_cell(Gtk::Menu* pMenu,
     _add_menu_separator(pMenu);
     if (not first_row) _add_menu_item(pMenu, find_action("table_row_up"));
     if (not last_row) _add_menu_item(pMenu, find_action("table_row_down"));
-    _add_menu_item(pMenu, find_action("table_rows_sort_descending"));
     _add_menu_item(pMenu, find_action("table_rows_sort_ascending"));
+    _add_menu_item(pMenu, find_action("table_rows_sort_descending"));
     _add_menu_separator(pMenu);
     _add_menu_item(pMenu, find_action("table_edit_properties"));
     _add_menu_item(pMenu, find_action("table_export"));
@@ -295,7 +289,7 @@ void CtMenu::_walk_menu_xml(Gtk::MenuShell* pMenuShell, xmlpp::Node* pNode)
         }
         else if (pNodeIter->get_name() == "menuitem") {
             CtMenuAction* pAction = find_action(get_attribute(pNodeIter, "action")->get_value());
-            _add_menu_item(pMenuShell, pAction);
+            if (pAction) _add_menu_item(pMenuShell, pAction);
         }
         else if (pNodeIter->get_name() == "separator") {
             _add_menu_separator(pMenuShell);
@@ -402,37 +396,58 @@ Gtk::MenuItem* CtMenu::_add_menu_item(Gtk::MenuShell* pMenuShell,
 {
     if (image && strlen(image)) {
         pMenuItem->set_name("ImageMenuItem");  // custom name to identify our "ImageMenuItems"
-        Gtk::Image *pIcon = Gtk::manage(new Gtk::Image{});
+        Gtk::Image* pIcon = Gtk::manage(new Gtk::Image{});
         pIcon->set_from_icon_name(image, Gtk::ICON_SIZE_MENU);
 
         // create a box to hold icon and label as GtkMenuItem derives from GtkBin and can only hold one child
-        auto const box = Gtk::manage(new Gtk::Box{Gtk::ORIENTATION_HORIZONTAL});
-        box->pack_start(*pIcon, false, false, 0);
-        box->pack_start(*pLabel, true, true, 0);
-
-        pMenuItem->add(*box);
+        static const int cImageMenuItemSpacing{8};
+        auto pBox = Gtk::manage(new Gtk::Box{Gtk::ORIENTATION_HORIZONTAL, cImageMenuItemSpacing});
+        if (pMenuItem->get_direction() == Gtk::TEXT_DIR_RTL) {
+            pBox->pack_end(*pIcon, false, false);
+            pBox->pack_end(*pLabel, true, true);
+        }
+        else {
+            pBox->pack_start(*pIcon, false, false);
+            pBox->pack_start(*pLabel, true, true);
+        }
+        pMenuItem->add(*pBox);
 
         // to fix image placement in MenuBar /context menu
         // based on inkscape: src/ui/desktop/menu-icon-shift.cpp
-        // we don't know which MenuItem will be first mapped, so connect  all of them
+        // we don't know which MenuItem will be first mapped, so connect all of them
+        auto f_calculate_image_shift = [](Gtk::MenuItem* menuItem)->int{
+            if (menuItem) {
+                if (auto box = dynamic_cast<Gtk::Box*>(menuItem->get_child())) {
+                    if (auto image = dynamic_cast<Gtk::Image*>(box->get_children()[0])) {
+                        auto allocation_menuitem = menuItem->get_allocation();
+                        auto allocation_image = image->get_allocation();
+                        if (menuItem->get_direction() == Gtk::TEXT_DIR_RTL) {
+                            return allocation_menuitem.get_width() - allocation_image.get_x() - allocation_image.get_width();
+                        }
+                        return -allocation_image.get_x();
+                    }
+                }
+            }
+            return 0;
+        };
         static std::list<sigc::connection>* static_map_connectons = new std::list<sigc::connection>();
         if (static_map_connectons != nullptr) { // if null then the fix was applied
-            static_map_connectons->push_back(pMenuItem->signal_map().connect([pMenuItem]
-            {
+            static_map_connectons->push_back(pMenuItem->signal_map().connect([f_calculate_image_shift, pMenuItem]{
                 spdlog::debug("shift images in MenuBar/context menu");
-                int shift = CtMenu::calculate_image_shift(pMenuItem);
+                const int shift = f_calculate_image_shift(pMenuItem);
                 if (shift != 0) {
                     auto provider = Gtk::CssProvider::create();
                     auto const screen = Gdk::Screen::get_default();
                     Gtk::StyleContext::add_provider_for_screen(screen, provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
                     if (pMenuItem->get_direction() == Gtk::TEXT_DIR_RTL) {
-                        provider->load_from_data("menuitem box image {margin-right:" + std::to_string(shift) + "px;}");
-                    } else {
-                        provider->load_from_data("menuitem box image {margin-left:" + std::to_string(shift) + "px;}");
+                        provider->load_from_data("menuitem box { margin-right:" + std::to_string(shift - cImageMenuItemSpacing) + "px; }");
+                    }
+                    else {
+                        provider->load_from_data("menuitem box { margin-left:" + std::to_string(shift + cImageMenuItemSpacing) + "px; }");
                     }
                 }
                 // we don't need to call this again, so kill all map connections
-                for (auto& connections: *static_map_connectons)
+                for (auto& connections : *static_map_connectons)
                     connections.disconnect();
                 delete static_map_connectons;
                 static_map_connectons = nullptr;
