@@ -1,7 +1,7 @@
 /*
  * ct_text_view.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -25,19 +25,21 @@
 #include "ct_text_view.h"
 #include "ct_actions.h"
 #include "ct_list.h"
+#include "ct_clipboard.h"
 
 std::unordered_map<std::string, GspellChecker*> CtTextView::_static_spell_checkers;
 
 CtTextView::CtTextView(CtMainWin* pCtMainWin)
  : _pCtMainWin{pCtMainWin}
+ , _pCtConfig{_pCtMainWin->get_ct_config()}
  , _columnEdit{*this}
 {
     set_smart_home_end(Gsv::SMART_HOME_END_AFTER);
     set_left_margin(7);
     set_right_margin(7);
-    set_insert_spaces_instead_of_tabs(_pCtMainWin->get_ct_config()->spacesInsteadTabs);
-    set_tab_width((guint)_pCtMainWin->get_ct_config()->tabsWidth);
-    if (_pCtMainWin->get_ct_config()->lineWrapping) {
+    set_insert_spaces_instead_of_tabs(_pCtConfig->spacesInsteadTabs);
+    set_tab_width((guint)_pCtConfig->tabsWidth);
+    if (_pCtConfig->lineWrapping) {
         set_wrap_mode(Gtk::WrapMode::WRAP_WORD_CHAR);
     }
     else {
@@ -50,6 +52,7 @@ CtTextView::CtTextView(CtMainWin* pCtMainWin)
     {
         set_border_window_size(textWinType, 1);
     }
+
     // column edit signals
     signal_event_after().connect([&](GdkEvent* pEvent){
         switch (pEvent->type) {
@@ -78,29 +81,54 @@ CtTextView::CtTextView(CtMainWin* pCtMainWin)
                 break;
         }
     }, false);
+    signal_focus_out_event().connect([&](GdkEventFocus* /*gdk_event*/){
+        _columnEdit.column_mode_off();
+        _set_highlight_current_line_enabled(false);
+        return false; /*propagate event*/
+    }, false);
+    signal_focus_in_event().connect([&](GdkEventFocus* /*gdk_event*/){
+        _set_highlight_current_line_enabled(true);
+        return false; /*propagate event*/
+    }, false);
+    _columnEdit.register_on_off_callback([&](const bool col_edit_on){
+        _set_highlight_current_line_enabled(not col_edit_on);
+        spdlog::debug("colMode {}", col_edit_on);
+    });
+}
+
+void CtTextView::_set_highlight_current_line_enabled(const bool enabled)
+{
+    if (enabled) {
+        const bool isRichTextOrTable{CtConst::RICH_TEXT_ID == _syntaxHighlighting or
+                                     CtConst::TABLE_CELL_TEXT_ID == _syntaxHighlighting};
+        set_highlight_current_line(isRichTextOrTable ? _pCtConfig->rtHighlCurrLine : _pCtConfig->ptHighlCurrLine);
+    }
+    else {
+        set_highlight_current_line(false);
+    }
 }
 
 void CtTextView::setup_for_syntax(const std::string& syntax)
 {
+    _syntaxHighlighting = syntax;
 #ifdef MD_AUTO_REPLACEMENT
-    if (_markdown_filter_active()) _md_handler->active(syntax == CtConst::RICH_TEXT_ID);
+    if (_markdown_filter_active()) _md_handler->active(_syntaxHighlighting == CtConst::RICH_TEXT_ID);
 #endif // MD_AUTO_REPLACEMENT
 
     std::string new_class;
-    if (CtConst::RICH_TEXT_ID == syntax)         { new_class = "ct-view-rich-text"; }
-    else if (CtConst::PLAIN_TEXT_ID == syntax)   { new_class = "ct-view-plain-text"; }
-    else                                         { new_class = "ct-view-code"; }
+    const bool isRichTextOrTable{CtConst::RICH_TEXT_ID == _syntaxHighlighting or
+                                 CtConst::TABLE_CELL_TEXT_ID == _syntaxHighlighting};
+    if (isRichTextOrTable)                                  { new_class = "ct-view-rich-text"; }
+    else if (CtConst::PLAIN_TEXT_ID == _syntaxHighlighting) { new_class = "ct-view-plain-text"; }
+    else                                                    { new_class = "ct-view-code"; }
 
     if (new_class != "ct-view-rich-text") get_style_context()->remove_class("ct-view-rich-text");
     if (new_class != "ct-view-plain-text") get_style_context()->remove_class("ct-view-plain-text");
     if (new_class != "ct-view-code") get_style_context()->remove_class("ct-view-code");
     get_style_context()->add_class(new_class);
 
-    if ( CtConst::RICH_TEXT_ID == syntax or
-         CtConst::TABLE_CELL_TEXT_ID == syntax )
-    {
-        set_highlight_current_line(_pCtMainWin->get_ct_config()->rtHighlCurrLine);
-        if (_pCtMainWin->get_ct_config()->rtShowWhiteSpaces) {
+    if (isRichTextOrTable) {
+        if (_pCtConfig->rtShowWhiteSpaces) {
             set_draw_spaces(Gsv::DRAW_SPACES_ALL & ~Gsv::DRAW_SPACES_NEWLINE);
         }
         else {
@@ -108,8 +136,7 @@ void CtTextView::setup_for_syntax(const std::string& syntax)
         }
     }
     else {
-        set_highlight_current_line(_pCtMainWin->get_ct_config()->ptHighlCurrLine);
-        if (_pCtMainWin->get_ct_config()->ptShowWhiteSpaces) {
+        if (_pCtConfig->ptShowWhiteSpaces) {
             set_draw_spaces(Gsv::DRAW_SPACES_ALL & ~Gsv::DRAW_SPACES_NEWLINE);
         }
         else {
@@ -149,31 +176,31 @@ void CtTextView::list_change_level(Gtk::TextIter iter_insert, const CtListInfo&
     _pCtMainWin->user_active() = false;
 
     int curr_offset = list_info.startoffs;
-    int end_offset = CtList{_pCtMainWin, get_buffer()}.get_multiline_list_element_end_offset(iter_insert, list_info);
+    int end_offset = CtList{_pCtConfig, get_buffer()}.get_multiline_list_element_end_offset(iter_insert, list_info);
     int curr_level = list_info.level;
     int next_level = level_increase ? curr_level+1 : curr_level-1;
     Gtk::TextIter iter_start = get_buffer()->get_iter_at_offset(curr_offset);
-    CtListInfo prev_list_info = CtList{_pCtMainWin, get_buffer()}.get_prev_list_info_on_level(iter_start, next_level);
-    // print prev_list_info
+    CtListInfo prev_list_info = CtList{_pCtConfig, get_buffer()}.get_prev_list_info_on_level(iter_start, next_level);
     if (list_info.type != CtListType::Todo) {
         int bull_offset = curr_offset + 3*list_info.level;
         int bull_idx;
-        if (list_info.type == CtListType::Bullet)
-        {
+        if (list_info.type == CtListType::Bullet) {
             if (prev_list_info and prev_list_info.type == CtListType::Bullet) {
-                bull_idx = prev_list_info.num;
+                bull_idx = prev_list_info.aux;
             }
             else {
-                int idx_old = list_info.num;
-                int idx_offset = idx_old - curr_level % (int)_pCtMainWin->get_ct_config()->charsListbul.size();
-                bull_idx = (next_level + idx_offset) % (int)_pCtMainWin->get_ct_config()->charsListbul.size();
+                int idx_old = list_info.aux;
+                int idx_offset = idx_old - curr_level % (int)_pCtConfig->charsListbul.size();
+                bull_idx = (next_level + idx_offset) % (int)_pCtConfig->charsListbul.size();
+                if (bull_idx < 0) bull_idx += _pCtConfig->charsListbul.size();
+                //spdlog::debug("idx_old={}, idx_offset={}, bull_idx={}", idx_old, idx_offset, bull_idx);
             }
-            replace_text(_pCtMainWin->get_ct_config()->charsListbul[(size_t)bull_idx], bull_offset, bull_offset+1);
+            replace_text(_pCtConfig->charsListbul[(size_t)bull_idx], bull_offset, bull_offset+1);
         }
         else if (list_info.type == CtListType::Number) {
             int this_num, index;
             if (prev_list_info and prev_list_info.type == CtListType::Number) {
-                this_num = prev_list_info.num + 1;
+                this_num = prev_list_info.num_seq + 1;
                 index = prev_list_info.aux;
             }
             else {
@@ -183,7 +210,7 @@ void CtTextView::list_change_level(Gtk::TextIter iter_insert, const CtListInfo&
                 index = (next_level + idx_offset) % CtConst::CHARS_LISTNUM.size();
             }
             Glib::ustring text_to = std::to_string(this_num) + Glib::ustring(1, CtConst::CHARS_LISTNUM[(size_t)index]) + CtConst::CHAR_SPACE;
-            replace_text(text_to, bull_offset, bull_offset + CtList{_pCtMainWin, get_buffer()}.get_leading_chars_num(list_info.type, list_info.num));
+            replace_text(text_to, bull_offset, bull_offset + CtList{_pCtConfig, get_buffer()}.get_leading_chars_num(list_info.type, list_info.num_seq));
         }
     }
     iter_start = get_buffer()->get_iter_at_offset(curr_offset);
@@ -199,7 +226,7 @@ void CtTextView::list_change_level(Gtk::TextIter iter_insert, const CtListInfo&
             end_offset -= 3;
             iter_start = get_buffer()->get_iter_at_offset(curr_offset+1);
         }
-        if (not CtList{_pCtMainWin, get_buffer()}.char_iter_forward_to_newline(iter_start) or not iter_start.forward_char()) {
+        if (not CtList{_pCtConfig, get_buffer()}.char_iter_forward_to_newline(iter_start) or not iter_start.forward_char()) {
             break;
         }
         curr_offset = iter_start.get_offset();
@@ -222,25 +249,39 @@ void CtTextView::for_event_after_double_click_button1(GdkEvent* event)
     window_to_buffer_coords(Gtk::TEXT_WINDOW_TEXT, (int)event->button.x, (int)event->button.y, x, y);
     Gtk::TextIter iter_start;
     get_iter_at_location(iter_start, x, y);
+    const auto pGutterLineNumbers = get_gutter(Gtk::TEXT_WINDOW_LEFT);
+    if (pGutterLineNumbers) {
+        const auto pGutterLNWindow = pGutterLineNumbers->get_window();
+        if (pGutterLNWindow and pGutterLNWindow->gobj() == event->button.window) {
+            // line number click
+            _pCtMainWin->apply_tag_try_automatic_bounds_paragraph(text_buffer, iter_start);
+            return;
+        }
+    }
     _pCtMainWin->apply_tag_try_automatic_bounds(text_buffer, iter_start);
 }
 
 // Called after every Triple Click with button 1
 void CtTextView::for_event_after_triple_click_button1(GdkEvent* event)
 {
-    auto text_buffer = get_buffer();
-    int x, y;
-    window_to_buffer_coords(Gtk::TEXT_WINDOW_TEXT, (int)event->button.x, (int)event->button.y, x, y);
-    Gtk::TextIter iter_start;
-    get_iter_at_location(iter_start, x, y);
-    _pCtMainWin->apply_tag_try_automatic_bounds_triple_click(text_buffer, iter_start);
+    if (_pCtConfig->tripleClickParagraph and
+        _pCtMainWin->curr_tree_iter().get_node_is_rich_text() and
+        get_todo_rotate_time() != event->button.time)
+    {
+        auto text_buffer = get_buffer();
+        int x, y;
+        window_to_buffer_coords(Gtk::TEXT_WINDOW_TEXT, (int)event->button.x, (int)event->button.y, x, y);
+        Gtk::TextIter iter_start;
+        get_iter_at_location(iter_start, x, y);
+        _pCtMainWin->apply_tag_try_automatic_bounds_paragraph(text_buffer, iter_start);
+    }
 }
 
 #ifdef MD_AUTO_REPLACEMENT
 bool CtTextView::_markdown_filter_active() {
-    bool is_active = _pCtMainWin->get_ct_config()->enableMdFormatting;
+    bool is_active = _pCtConfig->enableMdFormatting;
     if (is_active && !_md_handler) {
-        _md_handler = std::make_unique<CtMarkdownFilter>(std::make_unique<CtClipboard>(_pCtMainWin), get_buffer(), _pCtMainWin->get_ct_config());
+        _md_handler = std::make_unique<CtMarkdownFilter>(std::make_unique<CtClipboard>(_pCtMainWin), get_buffer(), _pCtConfig);
     }
     return is_active;
 }
@@ -271,7 +312,21 @@ void CtTextView::for_event_after_button_press(GdkEvent* event)
         // the issue: get_iter_at_position always gives iter, so we have to check if iter is valid
         Gdk::Rectangle iter_rect;
         get_iter_location(text_iter, iter_rect);
-        if (iter_rect.get_x() <= x && x <= (iter_rect.get_x() + iter_rect.get_width())) {
+        if (1 == event->button.button) {
+            const auto pGutterLineNumbers = get_gutter(Gtk::TEXT_WINDOW_LEFT);
+            if (pGutterLineNumbers) {
+                const auto pGutterLNWindow = pGutterLineNumbers->get_window();
+                if (pGutterLNWindow and pGutterLNWindow->gobj() == event->button.window) {
+                    // line number click
+                    _pCtMainWin->apply_tag_try_automatic_bounds(text_buffer, text_iter);
+                    return;
+                }
+            }
+        }
+        //spdlog::debug("x={} recx={} recw={}", x, iter_rect.get_x(), iter_rect.get_width());
+        if ( (iter_rect.get_width() >= 0/*LTR*/ and iter_rect.get_x() <= x and x <= (iter_rect.get_x() + iter_rect.get_width())) or
+             (iter_rect.get_width() < 0/*RTL*/ and (iter_rect.get_x() + iter_rect.get_width()) <= x and x <= iter_rect.get_x()) )
+        {
             auto tags = text_iter.get_tags();
             // check whether we are hovering a link
             for (auto& tag : tags) {
@@ -281,9 +336,9 @@ void CtTextView::for_event_after_button_press(GdkEvent* event)
                     return;
                 }
             }
-            if (CtList{_pCtMainWin, text_buffer}.is_list_todo_beginning(text_iter)) {
+            if (CtList{_pCtConfig, text_buffer}.is_list_todo_beginning(text_iter)) {
                 if (_pCtMainWin->get_ct_actions()->_is_curr_node_not_read_only_or_error()) {
-                    CtList{_pCtMainWin, text_buffer}.todo_list_rotate_status(text_iter);
+                    CtList{_pCtConfig, text_buffer}.todo_list_rotate_status(text_iter);
                     _todoRotateTime = event->button.time; // to prevent triple click
                 }
             }
@@ -305,11 +360,10 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
         return;
     }
     auto text_buffer = get_buffer();
-    auto config = _pCtMainWin->get_ct_config();
-    bool is_code = syntaxHighlighting != CtConst::RICH_TEXT_ID and syntaxHighlighting != CtConst::PLAIN_TEXT_ID;
+    bool is_code = syntaxHighlighting != CtConst::RICH_TEXT_ID and syntaxHighlighting != CtConst::PLAIN_TEXT_ID and syntaxHighlighting != CtConst::TABLE_CELL_TEXT_ID;
 
     if (not is_code and
-        config->autoSmartQuotes and
+        _pCtConfig->autoSmartQuotes and
         (event->key.keyval == GDK_KEY_quotedbl or event->key.keyval == GDK_KEY_apostrophe))
     {
         Gtk::TextIter iter_insert = text_buffer->get_insert()->get_iter();
@@ -318,13 +372,13 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
             Glib::ustring  start_char, char_0, char_1;
             if (event->key.keyval == GDK_KEY_quotedbl) {
                 start_char = CtConst::CHAR_DQUOTE;
-                char_0 = config->chars_smart_dquote[0];
-                char_1 = config->chars_smart_dquote[1];
+                char_0 = _pCtConfig->chars_smart_dquote[0];
+                char_1 = _pCtConfig->chars_smart_dquote[1];
             }
             else {
                 start_char = CtConst::CHAR_SQUOTE;
-                char_0 = config->chars_smart_squote[0];
-                char_1 = config->chars_smart_squote[1];
+                char_0 = _pCtConfig->chars_smart_squote[0];
+                char_1 = _pCtConfig->chars_smart_squote[1];
             }
             Gtk::TextIter iter_start = text_buffer->get_iter_at_offset(offset_1-1);
             int offset_0 = -1;
@@ -353,7 +407,7 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
             Gtk::TextIter iter_insert = text_buffer->get_insert()->get_iter();
             Gtk::TextIter iter_start = iter_insert;
             iter_start.backward_char();
-            CtListInfo list_info = CtList{_pCtMainWin, text_buffer}.get_paragraph_list_info(iter_start);
+            CtListInfo list_info = CtList{_pCtConfig, text_buffer}.get_paragraph_list_info(iter_start);
             if (list_info) {
                 text_buffer->insert(iter_insert, Glib::ustring(3*(1+(size_t)list_info.level), CtConst::CHAR_SPACE[0]));
             }
@@ -380,9 +434,9 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
             if (iter_start.get_char() != '\n') return;
             if (iter_start.backward_char() and iter_start.get_char() == '\n')
                 return; // former was an empty row
-            CtListInfo list_info = CtList{_pCtMainWin, text_buffer}.get_paragraph_list_info(iter_start);
+            CtListInfo list_info = CtList{_pCtConfig, text_buffer}.get_paragraph_list_info(iter_start);
             if (not list_info) {
-                if (config->autoIndent) {
+                if (_pCtConfig->autoIndent) {
                     iter_start = iter_insert;
                     Glib::ustring former_line_indent = _get_former_line_indentation(iter_start);
                     if (not former_line_indent.empty()) text_buffer->insert_at_cursor(former_line_indent);
@@ -391,7 +445,7 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
             }
             // possible enter on empty list element
             int insert_offset = iter_insert.get_offset();
-            int chars_to_startoffs = 1 + CtList{_pCtMainWin, text_buffer}.get_leading_chars_num(list_info.type, list_info.num) + 3*list_info.level;
+            int chars_to_startoffs = 1 + CtList{_pCtConfig, text_buffer}.get_leading_chars_num(list_info.type, list_info.num_seq) + 3*list_info.level;
             if ((insert_offset - list_info.startoffs) == chars_to_startoffs) {
                 if (iter_insert.ends_line()) {
                     // enter on empty list element
@@ -410,42 +464,42 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
                 // the list element was not empty, this list element should get a new list prefix
                 iter_start = iter_insert;
                 (void)iter_start.backward_chars(2);
-                list_info = CtList{_pCtMainWin, text_buffer}.get_paragraph_list_info(iter_start);
+                list_info = CtList{_pCtConfig, text_buffer}.get_paragraph_list_info(iter_start);
             }
             // list new element
             int curr_level = list_info.level;
             Glib::ustring pre_spaces = curr_level ? Glib::ustring((size_t)(3*curr_level), CtConst::CHAR_SPACE[0]) : "";
             if (list_info.type == CtListType::Bullet) {
-                text_buffer->insert(iter_insert, pre_spaces+config->charsListbul[(size_t)list_info.num]+CtConst::CHAR_SPACE);
+                text_buffer->insert(iter_insert, pre_spaces+_pCtConfig->charsListbul[(size_t)list_info.aux]+CtConst::CHAR_SPACE);
             }
             else if (list_info.type == CtListType::Todo) {
-                text_buffer->insert(iter_insert, pre_spaces+config->charsTodo[0]+CtConst::CHAR_SPACE);
+                text_buffer->insert(iter_insert, pre_spaces+_pCtConfig->charsTodo[0]+CtConst::CHAR_SPACE);
             }
             else {
-                int new_num = list_info.num + 1;
+                int new_num = list_info.num_seq + 1;
                 int index = list_info.aux;
                 text_buffer->insert(iter_insert, pre_spaces + std::to_string(new_num) + CtConst::CHARS_LISTNUM[(size_t)index] + CtConst::CHAR_SPACE);
                 new_num += 1;
                 iter_start = text_buffer->get_iter_at_offset(insert_offset);
-                CtList{_pCtMainWin, text_buffer}.char_iter_forward_to_newline(iter_start);
-                list_info = CtList{_pCtMainWin, text_buffer}.get_next_list_info_on_level(iter_start, curr_level);
+                CtList{_pCtConfig, text_buffer}.char_iter_forward_to_newline(iter_start);
+                list_info = CtList{_pCtConfig, text_buffer}.get_next_list_info_on_level(iter_start, curr_level);
                 // print list_info
                 while (list_info and list_info.type == CtListType::Number) {
                     iter_start = text_buffer->get_iter_at_offset(list_info.startoffs);
-                    int end_offset = CtList{_pCtMainWin, text_buffer}.get_multiline_list_element_end_offset(iter_start, list_info);
+                    int end_offset = CtList{_pCtConfig, text_buffer}.get_multiline_list_element_end_offset(iter_start, list_info);
                     Gtk::TextIter iter_end = text_buffer->get_iter_at_offset(end_offset);
-                    CtTextRange range = CtList{_pCtMainWin, text_buffer}.list_check_n_remove_old_list_type_leading(iter_start, iter_end);
+                    CtTextRange range = CtList{_pCtConfig, text_buffer}.list_check_n_remove_old_list_type_leading(iter_start, iter_end);
                     end_offset -= range.leading_chars_num;
                     text_buffer->insert(range.iter_start, std::to_string(new_num) + Glib::ustring(1, CtConst::CHARS_LISTNUM[(size_t)index]) + CtConst::CHAR_SPACE);
-                    end_offset += CtList{_pCtMainWin, text_buffer}.get_leading_chars_num(list_info.type, new_num);
+                    end_offset += CtList{_pCtConfig, text_buffer}.get_leading_chars_num(list_info.type, new_num);
                     iter_start = text_buffer->get_iter_at_offset(end_offset);
                     new_num += 1;
-                    list_info = CtList{_pCtMainWin, text_buffer}.get_next_list_info_on_level(iter_start, curr_level);
+                    list_info = CtList{_pCtConfig, text_buffer}.get_next_list_info_on_level(iter_start, curr_level);
                 }
             }
         }
         else { // keyname == CtConst::STR_KEY_SPACE
-            if (not is_code and config->enableSymbolAutoreplace and iter_start.backward_chars(2)) {
+            if (not is_code and _pCtConfig->enableSymbolAutoreplace and iter_start.backward_chars(2)) {
                 if (iter_start.get_char() == '>' and iter_start.backward_char()) {
                     if (iter_start.get_line_offset() == 0) {
                         // at line start
@@ -525,7 +579,7 @@ void CtTextView::for_event_after_key_press(GdkEvent* event, const Glib::ustring&
                 else if (iter_start.get_char() == ']' and iter_start.backward_char()) {
                     if (iter_start.get_line_offset() == 0 and iter_start.get_char() == '[') {
                         // "[] " becoming "☐ " at line start
-                        _special_char_replace(config->charsTodo[0], iter_start, iter_insert);
+                        _special_char_replace(_pCtConfig->charsTodo[0], iter_start, iter_insert);
                     }
                 }
                 else if (iter_start.get_char() == ':' and iter_start.backward_char()) {
@@ -553,8 +607,10 @@ void CtTextView::cursor_and_tooltips_handler(int x, int y)
     // the issue: get_iter_at_position always gives iter, so we have to check if iter is valid
     Gdk::Rectangle iter_rect;
     get_iter_location(text_iter, iter_rect);
-    if (iter_rect.get_x() <= x && x <= (iter_rect.get_x() + iter_rect.get_width())) {
-        if (CtList{_pCtMainWin, get_buffer()}.is_list_todo_beginning(text_iter)) {
+    if ( (iter_rect.get_width() >= 0/*LTR*/ and iter_rect.get_x() <= x and x <= (iter_rect.get_x() + iter_rect.get_width())) or
+         (iter_rect.get_width() < 0/*RTL*/ and (iter_rect.get_x() + iter_rect.get_width()) <= x and x <= iter_rect.get_x()) )
+    {
+        if (CtList{_pCtConfig, get_buffer()}.is_list_todo_beginning(text_iter)) {
             get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(Gdk::Cursor::create(Gdk::HAND2)); // Gdk::X_CURSOR doesn't work on Win
             set_tooltip_text("");
             return;
@@ -590,7 +646,6 @@ void CtTextView::cursor_and_tooltips_handler(int x, int y)
 
     if (_pCtMainWin->hovering_link_iter_offset() != hovering_link_iter_offset) {
         _pCtMainWin->hovering_link_iter_offset() = hovering_link_iter_offset;
-        // print "link", dad.hovering_link_iter_offset
     }
     if (_pCtMainWin->hovering_link_iter_offset() >= 0) {
         get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(Gdk::Cursor::create(Gdk::HAND2));
@@ -600,69 +655,73 @@ void CtTextView::cursor_and_tooltips_handler(int x, int y)
         set_tooltip_text(tooltip);
     }
     else {
-        get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(Gdk::Cursor::create(Gdk::XTERM));
-        set_tooltip_text("");
+        cursor_and_tooltips_reset();
     }
 }
 
+void CtTextView::cursor_and_tooltips_reset()
+{
+    _pCtMainWin->hovering_link_iter_offset() = -1;
+    get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(Gdk::Cursor::create(Gdk::XTERM));
+    set_tooltip_text("");
+}
+
 // Increase or Decrease Text Font
 void CtTextView::zoom_text(const bool is_increase, const std::string& syntaxHighlighting)
 {
     Glib::RefPtr<Gtk::StyleContext> context = get_style_context();
-    Pango::FontDescription fontDesc = context->get_font(context->get_state());
+    const Pango::FontDescription fontDesc = context->get_font(context->get_state());
     int size = fontDesc.get_size() / Pango::SCALE + (is_increase ? 1 : -1);
     if (size < 6) size = 6;
-    fontDesc.set_size(size * Pango::SCALE);
 
-    if (syntaxHighlighting == CtConst::RICH_TEXT_ID) {
-        auto pCtConfig = _pCtMainWin->get_ct_config();
-        pCtConfig->rtFont = CtFontUtil::get_font_str(fontDesc);
+    if (syntaxHighlighting == CtConst::RICH_TEXT_ID or syntaxHighlighting == CtConst::TABLE_CELL_TEXT_ID) {
+        _pCtConfig->rtFont = CtFontUtil::get_font_str(CtFontUtil::get_font_family(_pCtConfig->rtFont), size);
+        spdlog::debug("rtFont {}", _pCtConfig->rtFont);
         // also fix monospace font size
-        if (pCtConfig->msDedicatedFont and not pCtConfig->monospaceFont.empty()) {
-            Pango::FontDescription monoFontDesc(pCtConfig->monospaceFont);
+        if (_pCtConfig->msDedicatedFont and not _pCtConfig->monospaceFont.empty()) {
+            const Pango::FontDescription monoFontDesc(_pCtConfig->monospaceFont);
             int monoSize = monoFontDesc.get_size() / Pango::SCALE + (is_increase ? 1 : -1);
             if (monoSize < 6) monoSize = 6;
-            monoFontDesc.set_size(monoSize * Pango::SCALE);
 
-            pCtConfig->monospaceFont = CtFontUtil::get_font_str(monoFontDesc);
+            _pCtConfig->monospaceFont = CtFontUtil::get_font_str(CtFontUtil::get_font_family(_pCtConfig->monospaceFont), size);
             if (auto tag = get_buffer()->get_tag_table()->lookup(CtConst::TAG_ID_MONOSPACE)) {
-                tag->property_font() = pCtConfig->monospaceFont;
+                tag->property_font() = _pCtConfig->monospaceFont;
             }
         }
     }
     else if (syntaxHighlighting == CtConst::PLAIN_TEXT_ID) {
-        _pCtMainWin->get_ct_config()->ptFont = CtFontUtil::get_font_str(fontDesc);
+        _pCtConfig->ptFont = CtFontUtil::get_font_str(CtFontUtil::get_font_family(_pCtConfig->ptFont), size);
+        spdlog::debug("ptFont {}", _pCtConfig->ptFont);
     }
     else {
-        _pCtMainWin->get_ct_config()->codeFont = CtFontUtil::get_font_str(fontDesc);
+        _pCtConfig->codeFont = CtFontUtil::get_font_str(CtFontUtil::get_font_family(_pCtConfig->codeFont), size);
+        spdlog::debug("codeFont {}", _pCtConfig->codeFont);
     }
     _pCtMainWin->signal_app_apply_for_each_window([](CtMainWin* win) { win->update_theme(); });
 }
 
 void CtTextView::set_spell_check(bool allow_on)
 {
-    auto pCtConfig = _pCtMainWin->get_ct_config();
     auto gtk_view = GTK_TEXT_VIEW(gobj());
     auto gtk_buffer = gtk_text_view_get_buffer(gtk_view);
     auto gspell_buffer = gspell_text_buffer_get_from_gtk_text_buffer(gtk_buffer);
 
-    auto gspell_checker = _get_spell_checker(pCtConfig->spellCheckLang);
+    auto gspell_checker = _get_spell_checker(_pCtConfig->spellCheckLang);
     auto old_gspell_checker = gspell_text_buffer_get_spell_checker(gspell_buffer);
     if (gspell_checker != old_gspell_checker) {
         gspell_text_buffer_set_spell_checker(gspell_buffer, gspell_checker);
         // g_object_unref (gspell_checker); no need to unref because we keep it global
     }
     auto gspell_view = gspell_text_view_get_from_gtk_text_view(gtk_view);
-    gspell_text_view_set_inline_spell_checking(gspell_view, allow_on && pCtConfig->enableSpellCheck);
-    gspell_text_view_set_enable_language_menu(gspell_view, allow_on && pCtConfig->enableSpellCheck);
+    gspell_text_view_set_inline_spell_checking(gspell_view, allow_on && _pCtConfig->enableSpellCheck);
+    gspell_text_view_set_enable_language_menu(gspell_view, allow_on && _pCtConfig->enableSpellCheck);
 }
 
 void CtTextView::synch_spell_check_change_from_gspell_right_click_menu()
 {
-    auto pCtConfig = _pCtMainWin->get_ct_config();
     auto gtk_view = GTK_TEXT_VIEW(gobj());
     auto gtk_buffer = gtk_text_view_get_buffer(gtk_view);
-    if (not pCtConfig->enableSpellCheck) {
+    if (not _pCtConfig->enableSpellCheck) {
         return;
     }
     auto gspell_buffer = gspell_text_buffer_get_from_gtk_text_buffer(gtk_buffer);
@@ -672,8 +731,8 @@ void CtTextView::synch_spell_check_change_from_gspell_right_click_menu()
         return;
     }
     const std::string currSpellCheckLang = gspell_language_get_code(pGspellLang);
-    if (currSpellCheckLang != pCtConfig->spellCheckLang) {
-        pCtConfig->spellCheckLang = currSpellCheckLang;
+    if (currSpellCheckLang != _pCtConfig->spellCheckLang) {
+        _pCtConfig->spellCheckLang = currSpellCheckLang;
     }
 }
 
@@ -768,15 +827,15 @@ bool CtTextView::_apply_tag_try_link(Gtk::TextIter iter_end, int offset_cursor)
             }
         }
         int num_chars = iter_end.get_offset() - iter_start.get_offset();
-        if (num_chars > 4 and CtTextIterUtil::startswith_any(iter_start, CtConst::WEB_LINK_STARTERS)) {
+        if (_pCtConfig->urlAutoLink and num_chars > 4 and CtTextIterUtil::startswith_any(iter_start, CtConst::WEB_LINK_STARTERS)) {
             get_buffer()->select_range(iter_start, iter_end);
             Glib::ustring link_url = get_buffer()->get_text(iter_start, iter_end);
             if (not str::startswith(link_url, "htt") and !str::startswith(link_url, "ftp")) link_url = "http://" + link_url;
-            Glib::ustring property_value = std::string(CtConst::LINK_TYPE_WEBS) + CtConst::CHAR_SPACE + link_url;
+            Glib::ustring property_value = CtConst::LINK_TYPE_WEBS + CtConst::CHAR_SPACE + link_url;
             _pCtMainWin->get_ct_actions()->apply_tag(CtConst::TAG_LINK, property_value);
             tag_applied = true;
         }
-        else if (num_chars > 2 and CtTextIterUtil::get_is_camel_case(iter_start, num_chars)) {
+        else if (_pCtConfig->camelCaseAutoLink and num_chars > 2 and CtTextIterUtil::get_is_camel_case(iter_start, num_chars)) {
             if (apply_tag_try_node_name(iter_start, iter_end)) {
                 tag_applied = true;
             }
@@ -848,3 +907,43 @@ void CtTextView::_special_char_replace(Glib::ustring special_char, Gtk::TextIter
     _static_spell_checkers[lang] = gspell_checker;
     return gspell_checker;
 }
+
+void CtTextView::on_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context,
+                                       int x,
+                                       int y,
+                                       const Gtk::SelectionData& selection_data,
+                                       guint /*info*/,
+                                       guint time)
+{
+    bool success{false};
+    auto text_buffer = get_buffer();
+    int xb, yb;
+    window_to_buffer_coords(Gtk::TEXT_WINDOW_TEXT, x, y, xb, yb);
+    Gtk::TextIter text_iter;
+    int trailing;
+    get_iter_at_position(text_iter, trailing, xb, yb);
+    //spdlog::debug("targets: {}", str::join(context->list_targets(), ", "));
+    if (vec::exists(context->list_targets(), CtConst::TARGET_GTK_TEXT_BUFFER_CONTENTS) and text_buffer->get_has_selection()) {
+        int target_offset = text_iter.get_offset();
+        Gtk::TextIter sel_start, sel_end;
+        text_buffer->get_selection_bounds(sel_start, sel_end);
+        const int start_offset = sel_start.get_offset();
+        const int end_offset = sel_end.get_offset();
+        const int sel_size = end_offset - start_offset;
+        if (target_offset > start_offset) {
+            target_offset -= sel_size;
+        }
+        //spdlog::debug("drop {} from {} to {}", sel_size, start_offset, target_offset);
+        g_signal_emit_by_name(G_OBJECT(gobj()), "cut-clipboard");
+        text_iter = text_buffer->get_iter_at_offset(target_offset);
+        text_buffer->place_cursor(text_iter);
+        g_signal_emit_by_name(G_OBJECT(gobj()), "paste-clipboard");
+        success = true;
+    }
+    else if (vec::exists(context->list_targets(), CtConst::TARGET_URI_LIST)) {
+        text_buffer->place_cursor(text_iter);
+        CtClipboard{_pCtMainWin}.on_received_to_uri_list(selection_data, this, false/*forcePlain*/, true/*fromDragNDrop*/);
+        success = true;
+    }
+    context->drag_finish(success, false/*del*/, time);
+}
